/**
 * @license
 * Copyright (c) 2021 Handsoncode. All rights reserved.
 */
import { SimpleCellAddress } from './Cell';
import { RawCellContent } from './CellContentParser';
import { CellValue } from './CellValue';
import { Config } from './Config';
import { DependencyGraph } from './DependencyGraph';
import { Exporter } from './Exporter';
import { Maybe } from './Maybe';
import { Unparser } from './parser';
import { NamedExpressionOptions, NamedExpressions } from './NamedExpressions';
export interface SerializedNamedExpression {
    name: string;
    expression: RawCellContent;
    scope: Maybe<string>;
    options: Maybe<NamedExpressionOptions>;
}
export declare class Serialization {
    readonly dependencyGraph: DependencyGraph;
    readonly unparser: Unparser;
    readonly config: Config;
    readonly exporter: Exporter;
    constructor(dependencyGraph: DependencyGraph, unparser: Unparser, config: Config, exporter: Exporter);
    getCellFormula(address: SimpleCellAddress): Maybe<string>;
    getCellSerialized(address: SimpleCellAddress): RawCellContent;
    getCellValue(address: SimpleCellAddress): CellValue;
    getRawValue(address: SimpleCellAddress): RawCellContent;
    getSheetValues(sheet: number): CellValue[][];
    getSheetFormulas(sheet: number): Maybe<string>[][];
    genericSheetGetter<T>(sheet: number, getter: (address: SimpleCellAddress) => T): T[][];
    genericAllSheetsGetter<T>(sheetGetter: (sheet: number) => T): Record<string, T>;
    getSheetSerialized(sheet: number): RawCellContent[][];
    getAllSheetsValues(): Record<string, CellValue[][]>;
    getAllSheetsFormulas(): Record<string, Maybe<string>[][]>;
    getAllSheetsSerialized(): Record<string, RawCellContent[][]>;
    getAllNamedExpressionsSerialized(): SerializedNamedExpression[];
    withNewConfig(newConfig: Config, namedExpressions: NamedExpressions): Serialization;
}
