/**
 * @license
 * Copyright (c) 2021 Handsoncode. All rights reserved.
 */
import { AbsoluteCellRange } from '../AbsoluteCellRange';
import { DependencyGraph } from '../DependencyGraph';
import { MatrixSize } from '../Matrix';
import { Maybe } from '../Maybe';
import { InternalScalarValue } from './InterpreterValue';
export declare class ArrayData {
    readonly size: MatrixSize;
    readonly data: InternalScalarValue[][];
    _hasOnlyNumbers: boolean;
    constructor(size: MatrixSize, data: InternalScalarValue[][], _hasOnlyNumbers: boolean);
    range(): undefined;
    hasOnlyNumbers(): boolean;
    topLeftCorner(): Maybe<InternalScalarValue>;
    valuesFromTopLeftCorner(): InternalScalarValue[];
    iterateValuesFromTopLeftCorner(): IterableIterator<InternalScalarValue>;
    raw(): InternalScalarValue[][];
    rawNumbers(): number[][];
}
export declare class OnlyRangeData {
    readonly size: MatrixSize;
    readonly _range: AbsoluteCellRange;
    readonly dependencyGraph: DependencyGraph;
    data?: InternalScalarValue[][];
    _hasOnlyNumbers?: boolean;
    constructor(size: MatrixSize, _range: AbsoluteCellRange, dependencyGraph: DependencyGraph);
    raw(): InternalScalarValue[][];
    rawNumbers(): number[][];
    hasOnlyNumbers(): boolean;
    topLeftCorner(): Maybe<InternalScalarValue>;
    range(): AbsoluteCellRange;
    valuesFromTopLeftCorner(): InternalScalarValue[];
    iterateValuesFromTopLeftCorner(): IterableIterator<InternalScalarValue>;
    private ensureThatComputed;
    private computeDataFromDependencyGraph;
}
export declare type RangeData = ArrayData | OnlyRangeData;
export declare class SimpleRangeValue {
    readonly data: RangeData;
    private readonly _range?;
    get size(): MatrixSize;
    static onlyNumbersDataWithRange(data: number[][], size: MatrixSize, range: AbsoluteCellRange): SimpleRangeValue;
    static onlyNumbersDataWithoutRange(data: number[][], size: MatrixSize): SimpleRangeValue;
    static onlyRange(range: AbsoluteCellRange, dependencyGraph: DependencyGraph): SimpleRangeValue;
    static fromScalar(scalar: InternalScalarValue): SimpleRangeValue;
    constructor(data: RangeData, _range?: AbsoluteCellRange | undefined);
    width(): number;
    height(): number;
    raw(): InternalScalarValue[][];
    topLeftCornerValue(): Maybe<InternalScalarValue>;
    valuesFromTopLeftCorner(): InternalScalarValue[];
    iterateValuesFromTopLeftCorner(): IterableIterator<InternalScalarValue>;
    numberOfElements(): number;
    hasOnlyNumbers(): boolean;
    rawNumbers(): number[][];
    range(): Maybe<AbsoluteCellRange>;
    sameDimensionsAs(other: SimpleRangeValue): boolean;
}
