/**
 * @license
 * Copyright (c) 2021 Handsoncode. All rights reserved.
 */
import { SimpleCellAddress } from '../../Cell';
import { ProcedureAst } from '../../parser';
import { InternalScalarValue } from '../InterpreterValue';
import { ArgumentTypes, FunctionPlugin } from './FunctionPlugin';
export declare class ComplexPlugin extends FunctionPlugin {
    static implementedFunctions: {
        COMPLEX: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: string;
            })[];
        };
        IMABS: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMAGINARY: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMREAL: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMARGUMENT: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMCONJUGATE: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMCOS: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMCOSH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMCOT: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMCSC: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMCSCH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMSEC: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMSECH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMSIN: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMSINH: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMTAN: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMDIV: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMPRODUCT: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
            repeatLastArgs: number;
        };
        IMSUM: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
            repeatLastArgs: number;
        };
        IMSUB: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMEXP: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMLN: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMLOG10: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMLOG2: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMPOWER: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        IMSQRT: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
    };
    complex(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imabs(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imaginary(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imreal(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imargument(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imconjugate(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imcos(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imcosh(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imcot(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imcsc(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imcsch(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imsec(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imsech(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imsin(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imsinh(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imtan(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imdiv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    improduct(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imsum(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imsub(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imexp(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imln(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imlog10(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imlog2(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    impower(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    imsqrt(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
}
