/**
 * @license
 * Copyright (c) 2021 Handsoncode. All rights reserved.
 */
import { SimpleCellAddress } from '../../Cell';
import { ProcedureAst } from '../../parser';
import { InternalScalarValue } from '../InterpreterValue';
import { ArgumentTypes, FunctionPlugin } from './FunctionPlugin';
export declare class RandomPlugin extends FunctionPlugin {
    static implementedFunctions: {
        RAND: {
            method: string;
            parameters: never[];
            isVolatile: boolean;
        };
        RANDBETWEEN: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
            isVolatile: boolean;
        };
    };
    /**
     * Corresponds to RAND()
     *
     * Returns a pseudo-random floating-point random number
     * in the range [0,1).
     *
     * @param ast
     * @param formulaAddress
     */
    rand(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    randbetween(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
}
