/**
 * @license
 * Copyright (c) 2021 Handsoncode. All rights reserved.
 */
import { SimpleCellAddress } from '../../Cell';
import { ProcedureAst } from '../../parser';
import { InternalScalarValue } from '../InterpreterValue';
import { ArgumentTypes, FunctionPlugin } from './FunctionPlugin';
export declare class StatisticalPlugin extends FunctionPlugin {
    static implementedFunctions: {
        ERF: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                optionalArg?: undefined;
            } | {
                argumentType: ArgumentTypes;
                optionalArg: boolean;
            })[];
        };
        ERFC: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        'EXPON.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                greaterThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
                greaterThan?: undefined;
            })[];
        };
        FISHER: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan: number;
            }[];
        };
        FISHERINV: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        GAMMA: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        'GAMMA.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                greaterThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
                greaterThan?: undefined;
            })[];
        };
        GAMMALN: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                greaterThan: number;
            }[];
        };
        'GAMMA.INV': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                lessThan: number;
                greaterThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                minValue?: undefined;
                lessThan?: undefined;
            })[];
        };
        GAUSS: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        'BETA.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan?: undefined;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: number;
                greaterThan?: undefined;
            })[];
        };
        'BETA.INV': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan: number;
                maxValue: number;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                maxValue?: undefined;
                defaultValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                defaultValue: number;
                greaterThan?: undefined;
                maxValue?: undefined;
            })[];
        };
        'BINOM.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
                maxValue?: undefined;
            })[];
        };
        'BINOM.INV': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue?: undefined;
                greaterThan?: undefined;
                lessThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
                greaterThan?: undefined;
                lessThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan: number;
                minValue?: undefined;
                maxValue?: undefined;
            })[];
        };
        BESSELI: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
            })[];
        };
        BESSELJ: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
            })[];
        };
        BESSELK: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
            })[];
        };
        BESSELY: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
            })[];
        };
        'CHISQ.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            })[];
        };
        'CHISQ.DIST.RT': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue?: undefined;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            })[];
        };
        'CHISQ.INV': {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            }[];
        };
        'CHISQ.INV.RT': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue?: undefined;
            })[];
        };
        'F.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
            })[];
        };
        'F.DIST.RT': {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        'F.INV': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue?: undefined;
            })[];
        };
        'F.INV.RT': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue?: undefined;
            })[];
        };
        'WEIBULL.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                greaterThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
                greaterThan?: undefined;
            })[];
        };
        'POISSON.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
            })[];
        };
        'HYPGEOM.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                greaterThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
                greaterThan?: undefined;
            })[];
        };
        'T.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
            })[];
        };
        'T.DIST.2T': {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                minValue: number;
            }[];
        };
        'T.DIST.RT': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
            })[];
        };
        TDIST: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            })[];
        };
        'T.INV': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan: number;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                greaterThan?: undefined;
                lessThan?: undefined;
            })[];
        };
        'T.INV.2T': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan: number;
                maxValue: number;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                greaterThan?: undefined;
                maxValue?: undefined;
            })[];
        };
        'LOGNORM.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan: number;
            } | {
                argumentType: ArgumentTypes;
                greaterThan?: undefined;
            })[];
        };
        'LOGNORM.INV': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan: number;
            } | {
                argumentType: ArgumentTypes;
                greaterThan?: undefined;
                lessThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan?: undefined;
            })[];
        };
        'NORM.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
            })[];
        };
        'NORM.INV': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan: number;
            } | {
                argumentType: ArgumentTypes;
                greaterThan?: undefined;
                lessThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan?: undefined;
            })[];
        };
        'NORM.S.DIST': {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        'NORM.S.INV': {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan: number;
            }[];
        };
        PHI: {
            method: string;
            parameters: {
                argumentType: ArgumentTypes;
            }[];
        };
        'NEGBINOM.DIST': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                maxValue: number;
            } | {
                argumentType: ArgumentTypes;
                minValue?: undefined;
                maxValue?: undefined;
            })[];
        };
        'CONFIDENCE.NORM': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan: number;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan?: undefined;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                greaterThan?: undefined;
                lessThan?: undefined;
            })[];
        };
        'CONFIDENCE.T': {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan: number;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
                lessThan?: undefined;
                minValue?: undefined;
            } | {
                argumentType: ArgumentTypes;
                minValue: number;
                greaterThan?: undefined;
                lessThan?: undefined;
            })[];
        };
        STANDARDIZE: {
            method: string;
            parameters: ({
                argumentType: ArgumentTypes;
                greaterThan?: undefined;
            } | {
                argumentType: ArgumentTypes;
                greaterThan: number;
            })[];
        };
    };
    static aliases: {
        NEGBINOMDIST: string;
        EXPONDIST: string;
        BETADIST: string;
        NORMDIST: string;
        NORMINV: string;
        NORMSDIST: string;
        NORMSINV: string;
        LOGNORMDIST: string;
        LOGINV: string;
        TINV: string;
        HYPGEOMDIST: string;
        POISSON: string;
        WEIBULL: string;
        FINV: string;
        FDIST: string;
        CHIDIST: string;
        CHIINV: string;
        GAMMADIST: string;
        'GAMMALN.PRECISE': string;
        GAMMAINV: string;
        BETAINV: string;
        BINOMDIST: string;
        CONFIDENCE: string;
        CRITBINOM: string;
        WEIBULLDIST: string;
        TINV2T: string;
        TDISTRT: string;
        TDIST2T: string;
        FINVRT: string;
        FDISTRT: string;
        CHIDISTRT: string;
        CHIINVRT: string;
        LOGNORMINV: string;
        POISSONDIST: string;
    };
    erf(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    erfc(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    expondist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    fisher(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    fisherinv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    gamma(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    gammadist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    gammaln(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    gammainv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    gauss(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    betadist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    betainv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    binomialdist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    binomialinv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    besselifn(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    besseljfn(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    besselkfn(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    besselyfn(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    chisqdist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    chisqdistrt(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    chisqinv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    chisqinvrt(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    fdist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    fdistrt(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    finv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    finvrt(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    weibulldist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    poissondist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    hypgeomdist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    tdist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    tdist2t(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    tdistrt(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    tdistold(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    tinv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    tinv2t(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    lognormdist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    lognorminv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    normdist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    norminv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    normsdist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    normsinv(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    phi(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    negbinomdist(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    confidencenorm(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    confidencet(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
    standardize(ast: ProcedureAst, formulaAddress: SimpleCellAddress): InternalScalarValue;
}
